use std::{collections::HashMap, rc::Rc};

use crate::{
    DeBrujinAst, PrimitiveType, Type,
    inference::infer_type,
    parse::{Ast, Constant},
    types::{TaggedType, TypeTag},
};

#[test]
fn infer_add_nat() {
    let ast: DeBrujinAst = Ast::Application(
        Box::new(Ast::Application(
            Box::new(Ast::Variable("add".to_string())),
            Box::new(Ast::Constant(Constant::Nat(5))),
        )),
        Box::new(Ast::Constant(Constant::Nat(7))),
    )
    .into();

    let mut gamma = HashMap::new();
    gamma.insert(
        "add".to_string(),
        crate::types::TaggedType::Tagged(
            TypeTag::Num,
            "a".to_string(),
            Box::new(TaggedType::Concrete(Type::Arrow(
                Box::new(Type::Generic("a".to_string())),
                Box::new(Type::Arrow(
                    Box::new(Type::Generic("a".to_string())),
                    Box::new(Type::Generic("a".to_string())),
                )),
            ))),
        ),
    );

    let infered = infer_type(&gamma, ast).unwrap();
    assert_eq!(
        infered,
        TaggedType::Concrete(Type::Primitive(PrimitiveType::Nat))
    );
}

#[test]
fn infer_add_nat_partial() {
    let ast: DeBrujinAst = Ast::Application(
        Box::new(Ast::Variable("add".to_string())),
        Box::new(Ast::Constant(Constant::Nat(5))),
    )
    .into();

    let mut gamma = HashMap::new();
    gamma.insert(
        "add".to_string(),
        crate::types::TaggedType::Tagged(
            TypeTag::Num,
            "a".to_string(),
            Box::new(TaggedType::Concrete(Type::Arrow(
                Box::new(Type::Generic("a".to_string())),
                Box::new(Type::Arrow(
                    Box::new(Type::Generic("a".to_string())),
                    Box::new(Type::Generic("a".to_string())),
                )),
            ))),
        ),
    );

    let infered = infer_type(&gamma, ast).unwrap();
    assert_eq!(
        infered,
        TaggedType::Concrete(Type::arrow(PrimitiveType::Nat, PrimitiveType::Nat))
    );
}
